#!/usr/bin/env python

"""

GrabberSnap
SegPhault (Ryan Paul) - 12/06/2008

"""

import os
import gtk, gobject
import goocanvas, cairo, canvas

SAMPLE_SCREENSHOT = os.path.expanduser("~/screenshots/screenshot111.png")
SAMPLE_TEXT = "<span size='x-large' font_family='Sans' weight='heavy'>This is a test</span>"

class GrabberSnapCanvas(goocanvas.Canvas):
  def __init__(self, image = None):
    goocanvas.Canvas.__init__(self)

    if image: self.set_base_image(image)
    self.add_selection_rect(x=1, y=1, width=200, height=200)

    self += canvas.Label(
        fill_color="white", shadow=True,
        shadow_color=canvas.convert_color("black", 100),
        text=SAMPLE_TEXT, use_markup=True)
  
  def add_selection_rect(self, **args):
    self.selection_rect = canvas.ResizableRect(**args)
    self.selection_rect.props.line_dash = goocanvas.LineDash([2.0, 2.0, 2.0, 2.0, 2.0])
    self.selection_rect.props.fill_color_rgba = canvas.convert_color("gray", 80)

    self += self.selection_rect

  def zoom(self, amount):
    self.set_scale(amount)

  def crop(self):
    b = self.selection_rect.get_bounds()
    pb = self.base_image_data.subpixbuf(int(b.x1), int(b.y1),
      int(self.selection_rect.props.width),
      int(self.selection_rect.props.height))
    self.update_base_image(pb)

  def capture(self):
    x, y, w, h = self.get_bounds()
    surface = cairo.ImageSurface(cairo.FORMAT_ARGB32, int(w), int(h))
    context = cairo.Context(surface)
    context.rectangle(0, 0, 1000, 1000)
    context.set_source_rgb(1, 1, 1)
    context.fill()

    self.render(context)
    surface.write_to_png("/home/segphault/test.png")

  def set_base_image(self, image):
    if isinstance(image, str):
      self.base_image = goocanvas.Image(x=0, y=0)
      self.update_base_image(gtk.gdk.pixbuf_new_from_file(image))
      self += self.base_image

  def update_base_image(self, image_data):
    self.base_image_data = image_data
    self.base_image.props.pixbuf = image_data
    self.set_bounds(0, 0, self.base_image.props.width, self.base_image.props.height)

  def __iadd__(self, item):
    self.get_root_item().add_child(item)
    return self

class GrabberSnap(gtk.Window):
  def __init__(self):
    gtk.Window.__init__(self)
    self.resize(500, 500)
    self.connect("destroy", gtk.main_quit)

    self.canvas = GrabberSnapCanvas(SAMPLE_SCREENSHOT)
    
    scroll = gtk.ScrolledWindow()
    scroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
    scroll.add(self.canvas)
    
    button1 = gtk.Button("Capture")
    button1.connect("clicked", lambda *a: self.canvas.capture())
    
    button2 = gtk.Button("Crop")
    button2.connect("clicked", lambda *a: self.canvas.crop())

    zoom_slider = gtk.HScale(gtk.Adjustment(1, 0.1, 3, 0.1))
    zoom_slider.connect("change-value", self.on_zoom_change)

    statusbar = gtk.Statusbar()
    statusbar.pack_start(zoom_slider)

    vb = gtk.VBox(spacing=5)
    vb.pack_start(scroll, True, True)
    vb.pack_start(button2, False, False)
    vb.pack_start(button1, False, False)
    vb.pack_start(statusbar, False, False)

    self.add(vb)
    self.show_all()

  def on_zoom_change(self, w, t, v):
    self.canvas.zoom(v)

if __name__ == "__main__":
  w = GrabberSnap()
  gtk.main()
