#!/usr/bin/env python

"""

GrabberSnap - Screenshot module
SegPhault (Ryan Paul) - 12/07/2008

"""

from gtk import gdk

"""
import wnck

def get_visible_windows():
  wnck.screen_get_default().force_update()
  for w in wnck.screen_get_default().get_windows():
    if not w.is_skip_tasklist() and not w.is_minimized(): yield w
"""

def window_to_pixbuf(fn, win):
  w, h = win.get_size()
  pb = gdk.Pixbuf(gdk.COLORSPACE_RGB, True, 8, w, h)
  pb.get_from_drawable(win, w.get_colormap(), 0, 0, 0, 0, w, h)
  return pb

def region_to_pixbuf(fn, x, y, w, h):
  root = gdk.get_default_root_window()
  pb = gdk.Pixbuf(gdk.COLORSPACE_RGB, True, 8, w, h)
  pb.get_from_drawable(root, root.get_colormap(), x, y, 0, 0, w, h)
  return pb

class ScreenRegionGrabber:
  def __init__(self):
    self.root = gdk.get_default_root_window()
    self.gc = self.setup_canvas(self.root)
    self.dragging = False
    self.region = None

  def on_finish_capture(self, x, y, w, h):
    pass

  def capture(self, on_finish = None):
    if on_finish: self.on_finish_capture = on_finish
    gdk.pointer_grab(self.root, False,
      gdk.POINTER_MOTION_MASK |
      gdk.BUTTON1_MOTION_MASK |
      gdk.BUTTON_RELEASE_MASK |
      gdk.BUTTON1_MASK, None, gdk.Cursor(gdk.CROSSHAIR))

    gdk.event_handler_set(self.on_event)

  def setup_canvas(self, win):
    gc = win.new_gc()
    gc.set_line_attributes(1, gdk.LINE_ON_OFF_DASH, gdk.CAP_BUTT, gdk.CAP_BUTT)
    gc.set_rgb_bg_color(gdk.color_parse("black"))
    gc.set_rgb_fg_color(gdk.color_parse("white"))
    gc.set_subwindow(gdk.INCLUDE_INFERIORS)
    gc.set_function("xor")
    return gc

  def on_event(self, e):
    if e.type == gdk.BUTTON_PRESS:
      self.dragging = True
      self.drag_start = e.x, e.y
      
    if e.type == gdk.BUTTON_RELEASE:
      self.root.draw_rectangle(self.gc, 0, *self.region)
      self.dragging = False
      gdk.pointer_ungrab()
      self.on_finish_capture(self, *self.region)

    if e.type == gdk.MOTION_NOTIFY:
      if self.dragging:
        x, y = self.drag_start
        if self.region: self.root.draw_rectangle(self.gc, 0, *self.region)

        self.region = (int(min(x, e.x)), int(min(y, e.y)),
          int(e.x - x) if x < e.x else ~int(e.x - x),
          int(e.y - y) if y < e.y else ~int(e.y - y))

        self.root.draw_rectangle(self.gc, 0, *self.region)
        gdk.flush()

    gtk.main_do_event(e)

if __name__ == "__main__":
  import gtk
  w = gtk.Window()
  w.connect("destroy", gtk.main_quit)
  w.show_all()
  
  def test(region, x, y, w, h): print x,y,w,h
  ScreenRegionGrabber().capture(test)
  
  gtk.main()
